/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                               Initial Setup                              */
/*                      Populating Partitioned DB with Data                 */
/****************************************************************************/

set noexec off
set xact_abort on
set nocount on
go

use PartitionedDB
go

if not exists(select * from sys.databases where name = 'NonPartitionedDB')
begin
	raiserror('Please create NonPartitionedDB with the data first',16,1)
	set noexec on
end
go

if not exists((select * from NonPartitionedDB.sys.tables t join NonPartitionedDB.sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='Orders'))
begin
	raiserror('Please create PartitionedDB.Orders table first',16,1)
	set noexec on
end
go

if exists(select * from sys.views v join sys.schemas s on v.schema_Id = s.schema_Id where s.name = 'dbo' and v.name='Orders') drop view dbo.Orders;
if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='Orders2017_11') drop table dbo.Orders2017_11;
if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='Orders2017_10') drop table dbo.Orders2017_10;
if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='Orders2017') drop table dbo.Orders2017;
if exists(select * from sys.partition_schemes where name = 'psOrders2017') drop partition scheme psOrders2017;
if exists(select * from sys.partition_functions where name = 'pfOrders2017') drop partition function pfOrders2017;

create table dbo.Orders2017_11
(
	OrderDate datetime2(0) not null,
	OrderId int identity(1,1) not null,
	WarehouseId int not null,	
	CustomerId int not null, 
	ZipCode char(5) not null,
	Amount money not null,
	DeliveryInstructions varchar(1000) null,

	constraint CHK_Orders_2017_11
	check (OrderDate >= '2017-11-01' and OrderDate < '2017-12-01')
) on [CurrentOrders];

create unique clustered index IDX_Orders2017_11_OrderDate_OrderId
on dbo.Orders2017_11(OrderDate,OrderId)
on [CurrentOrders]
go

create table dbo.Orders2017_10
(
	OrderDate datetime2(0) not null,
	OrderId int not null,
	WarehouseId int not null,	
	CustomerId int not null, 
	ZipCode char(5) not null,
	Amount money not null,
	DeliveryInstructions varchar(1000) null,
	constraint CHK_Orders_2017_10
	check (OrderDate >= '2017-10-01' and OrderDate < '2017-11-01')
) on [RecentOrders];

create unique clustered index IDX_Orders2017_10_OrderDate_OrderId
on dbo.Orders2017_10(OrderDate,OrderId)
with (data_compression=row)
on [RecentOrders]
go

create partition function pfOrders2017(datetime2(0))
as range right for values
('2017-02-01','2017-03-01','2017-04-01','2017-05-01'
,'2017-06-01','2017-07-01','2017-08-01','2017-09-01'
,'2017-10-01','2017-11-01','2017-12-01','2018-01-01');

create partition scheme psOrders2017 
as partition pfOrders2017
to
([OldOrders],[OldOrders],[OldOrders],[OldOrders],[OldOrders],[OldOrders]
,[OldOrders],[OldOrders]
,[RecentOrders],[RecentOrders],[RecentOrders],[RecentOrders],[RecentOrders]);
go

create table dbo.Orders2017
(
	OrderDate datetime2(0) not null,
	OrderId int not null,
	WarehouseId int not null,	
	CustomerId int not null, 
	ZipCode char(5) not null,
	Amount money not null,
	constraint CHK_Orders_2017_01_09
	check (OrderDate >= '2017-01-01' and OrderDate < '2017-10-01'),
	constraint CHK_Orders_2017_01_10
	check (OrderDate >= '2017-01-01' and OrderDate < '2017-11-01'),
	constraint CHK_Orders_2017_01_11
	check (OrderDate >= '2017-01-01' and OrderDate < '2017-12-01'),
	constraint CHK_Orders_2017
	check (OrderDate >= '2017-01-01' and OrderDate < '2018-01-01')
) on psOrders2017(OrderDate);

create unique clustered index IDX_Orders2017_OrderDate_OrderId
on dbo.Orders2017(OrderDate,OrderId)
with (data_compression=page)
on psOrders2017(OrderDate);
go


set identity_insert dbo.Orders2017_11 on
insert into dbo.Orders2017_11 with (TABLOCKX)
		(OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount, DeliveryInstructions)
	select OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount, DeliveryInstructions
	from NonPartitionedDB.dbo.Orders with (TABLOCK)
	where OrderDate >= '2017-11-01'
set identity_insert dbo.Orders2017_11 off
go
 
insert into dbo.Orders2017_10 with (TABLOCKX)
		(OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount, DeliveryInstructions)
	select OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount, DeliveryInstructions
	from NonPartitionedDB.dbo.Orders with (TABLOCK)
	where OrderDate >= '2017-10-01' and OrderDate < '2017-11-01';


insert into dbo.Orders2017 with (TABLOCKX)
		(OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount)
	select OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount
	from NonPartitionedDB.dbo.Orders with (TABLOCK)
	where OrderDate < '2017-10-01';

create nonclustered index IDX_Orders2017_11_WarehouseId_OrderDate 
on dbo.Orders2017_11(WarehouseId, OrderDate)
on [CurrentOrders];

create nonclustered index IDX_Orders2017_10_WarehouseId_OrderDate 
on dbo.Orders2017_10(WarehouseId, OrderDate)
include(ZipCode, Amount)
with (data_compression=row)
on [RecentOrders];

create nonclustered index IDX_Orders2017_WarehouseId_OrderDate 
on dbo.Orders2017(WarehouseId, OrderDate)
include(ZipCode, Amount)
with (data_compression=page)
on psOrders2017(OrderDate);
go


update statistics dbo.Orders2017_11 with fullscan;
update statistics dbo.Orders2017_10 with fullscan;
update statistics dbo.Orders2017 with fullscan;
go

create view dbo.Orders(OrderDate, OrderId, WarehouseId, CustomerId
	,ZipCode, Amount, DeliveryInstructions)
as
	select OrderDate, OrderId, WarehouseId, CustomerId
		,ZipCode, Amount, DeliveryInstructions
	from dbo.Orders2017_11

	union all

	select OrderDate, OrderId, WarehouseId, CustomerId
		,ZipCode, Amount, DeliveryInstructions
	from dbo.Orders2017_10

	union all

	select OrderDate, OrderId, WarehouseId, CustomerId
		,ZipCode, Amount, NULL as DeliveryInstructions
	from dbo.Orders2017
go

