/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                        Benefits of Data Partitioning                     */
/*                     Better Statistics / Execution Plans                  */
/****************************************************************************/

set nocount on
go

-- Total Number of Orders: DBs are the same
select year(OrderDate), month(OrderDate), count(*) as [Non Partitioned DB Order Count]
from NonPartitionedDb.dbo.Orders with (tablock)
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

select year(OrderDate), month(OrderDate), count(*) as [Partitioned DB Order Count]
from PartitionedDB.dbo.Orders with (tablock)
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

-- Total Number of Orders Per Month in WarehouseId = 10
select year(OrderDate), month(OrderDate), count(*) as [Non Partitioned DB Order Count]
from NonPartitionedDb.dbo.Orders with (tablock)
where WarehouseId = 10
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

select year(OrderDate), month(OrderDate), count(*) as [Partitioned DB Order Count]
from PartitionedDB.dbo.Orders with (tablock)
where WarehouseId = 10
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

/* Statistics is up to date in every index but you can update it with:
update NonPartitionedDb.dbo.Orders with fullscan;
update statistics dbo.Orders2017_10 with fullscan;
update statistics dbo.Orders2017_09 with fullscan;
update statistics dbo.Orders2017 with fullscan;
*/

-- Statistics on the index
use PartitionedDB
go

dbcc show_statistics('dbo.Orders2017_11','IDX_Orders2017_11_WarehouseId_OrderDate')
go

use NonPartitionedDB
go
dbcc show_statistics('dbo.Orders','IDX_Orders_WarehouseId_OrderDate')
go

set statistics io on
select top 100 *
from NonPartitionedDb.dbo.Orders
where WarehouseId = 10 and OrderDate >= '2017-11-01'
order by Amount desc
option (maxdop 1)

select top 100 *
from PartitionedDb.dbo.Orders
where WarehouseId = 10 and OrderDate >= '2017-11-01'
order by Amount desc
option (maxdop 1)
set statistics io off
go

