/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                        Benefits of Data Partitioning                     */
/*                      Disaster Recovery - Poecemeal Restore               */
/*						  Requires Enterprise Edition                       */
/****************************************************************************/
use master
go

/*
backup database [NonPartitionedDB] 
to disk = N'C:\DB\Backup\NonPartitionedDB.bak' with format, init,
name = N'NonPartitionedDB-Full Database Backup', 
skip, norewind, nounload, compression,  stats = 5;
go

backup database [PartitionedDB] 
to disk = N'C:\DB\Backup\PartitionedDB.bak' with format, init,
name = N'PartitionedDB-Full Database Backup', 
skip, norewind, nounload, compression,  stats = 5;
go
*/


-- Let's consider that we have DATA drive failure.
-- First, let's take tail-log backup for both databases

backup log PartitionedDB 
to disk = N'C:\DB\Backup\PartitionedDB-tail.trn' 
with no_truncate, format, init, 
name = N'PartitionedDB-Tail-Log Backup', 
skip, norewind, nounload, compression,  stats = 5;
go

backup log NonPartitionedDB 
to disk = N'C:\DB\Backup\NonPartitionedDB-tail.trn' 
with no_truncate, format, init, 
name = N'NonPartitionedDB-Tail-Log Backup', 
skip, norewind, nounload, compression,  stats = 5;
go

-- Next, let's drop databases
alter database NonPartitionedDb set single_user with rollback immediate;
alter database PartitionedDb set single_user with rollback immediate;
drop database NonPartitionedDb;
drop database PartitionedDb;
go

-- First, let's restore non-partitioned DB to different drive 
restore database NonPartitionedDb 
from disk = N'C:\DB\Backup\NonPartitionedDB.bak' with file = 1,
move N'NonPartitionedDB' to N'C:\DB2\NonPartitionedDB.mdf',
norecovery, nounload,  partial, stats = 10;

restore log NonPartitionedDb 
from disk = N'C:\DB\Backup\NonPartitionedDB-tail.trn' 
with nounload, norecovery, stats = 10;

restore database NonPartitionedDb with recovery;
go

-- Second, let's restore PartitionedDB OLTP Filegroup
restore database PartitionedDB 
filegroup = 'Primary', filegroup = 'CurrentOrders', filegroup = 'RecentOrders'
from disk = N'C:\DB\Backup\PartitionedDB.bak' with file = 1, 
move N'PartitionedDB' to N'C:\DB2\PartitionedDB.mdf', 
move N'PartitionedDB_CurrentOrders' to N'C:\DB2\PartitionedDB_CurrentOrders.ndf', 
move N'PartitionedDB_RecentOrders' to N'C:\DB2\PartitionedDB_RecentOrders.ndf', 
norecovery, nounload,  partial, stats = 10;

restore log PartitionedDB 
from disk = N'C:\DB\Backup\PartitionedDB-tail.trn' 
with nounload, norecovery, stats = 10;

restore database PartitionedDB with recovery;
go

select file_id, name, state_desc, physical_name
from PartitionedDB.sys.database_files
go

-- Success
select top 10 *
from PartitionedDB.dbo.Orders2017_11
go

-- Fail
select top 10 *
from PartitionedDB.dbo.Orders 
where OrderDate >= '2017-01-01'
order by OrderDate;
go

-- Restoring Old Data FG. This will work with both, R/O and R/W filegrouos
restore database PartitionedDB 
filegroup = 'OldOrders', filegroup = 'CCIOrders'
from disk = N'C:\DB\Backup\PartitionedDB.bak' with file = 1,  
move N'PartitionedDB_OldOrders' to N'C:\DB2\PartitionedDB_OldOrdersPartitionedDB_RecentOrders.ndf', 
move N'PartitionedDB_CCIOrders' to N'C:\DB2\PartitionedDB_CCIOrders.ndf', 
norecovery,  nounload,  stats = 5;

restore log PartitionedDB 
from disk = N'C:\DB\Backup\PartitionedDB-tail.trn' 
with nounload, recovery, stats = 10;
go

select file_id, name, state_desc, physical_name
from PartitionedDB.sys.database_files
go

-- Success
select top 10 *
from PartitionedDB.dbo.Orders 
where OrderDate >= '2017-01-01'
order by OrderDate;
go