/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                           Partitioned Tables                             */
/*                               Overview                                   */
/****************************************************************************/

use PartitionedDB
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='Transactions2017') drop table dbo.Transactions2017;
if exists(select * from sys.partition_schemes where name = 'psTransactions2017') drop partition scheme psTransactions2017;
if exists(select * from sys.partition_functions where name = 'pfTransactions2017') drop partition function pfTransactions2017;

create partition function pfTransactions2017(datetime2(0))
as range right for values
('2017-02-01','2017-03-01','2017-04-01','2017-05-01'
,'2017-06-01','2017-07-01','2017-08-01','2017-09-01'
,'2017-10-01','2017-11-01','2017-12-01','2018-01-01');

create partition scheme psTransactions2017 
as partition pfTransactions2017
to
([RecentOrders],[RecentOrders],[RecentOrders],[RecentOrders],[RecentOrders],[RecentOrders]
,[RecentOrders],[RecentOrders]
,[CurrentOrders],[CurrentOrders],[CurrentOrders],[CurrentOrders],[CurrentOrders]);
go

create table dbo.Transactions2017
(
	TranDate datetime2(0) not null,
	TranId int not null,
	TranCode char(1) not null, 
	CustomerId int not null,
	Data varchar(1024) null,
) on psTransactions2017(TranDate);

create unique clustered index IDX_Transactions2017_TranDate_TranId
on dbo.Transactions2017(TranDate,TranId)
with 
(
	data_compression=page on partitions(1 to 9),
	data_compression=row on partitions(10 to 13)
)
on psTransactions2017(TranDate);

create nonclustered index IDX_Transactions2017_CustomerId
on dbo.Transactions2017(CustomerId)
with 
(
	data_compression=page on partitions(1 to 9),
	data_compression=row on partitions(10 to 13)
)
on psTransactions2017(TranDate);
go

select 
	i.name, ic.index_column_id, c.name, ic.key_ordinal
	,ic.is_included_column, ic.partition_ordinal 
from 
	sys.indexes i  left join sys.index_columns ic on 
		i.object_id = ic.object_id and 
		i.index_id = ic.index_id
	left join sys.columns c on 
		ic.object_id = c.object_id and 
		ic.column_id = c.column_id
where
	i.object_id = object_id(N'dbo.Transactions2017') and 
	i.name = 'IDX_Transactions2017_CustomerId'
go

insert into dbo.Transactions2017(TranDate, TranId, TranCode, CustomerId) 
	select OrderDate, OrderId, 'I', CustomerId
	from NonPartitionedDB.dbo.Orders
	where OrderId % 100 = 0;
go

select * from sys.partition_functions;

select v.boundary_id, f.boundary_value_on_right, v.value
from sys.partition_range_values v join sys.partition_functions f on 
	v.function_id = f.function_id
where f.name = 'pfTransactions2017'
order by v.boundary_id;

select 
	t.object_id, i.index_id, i.name as [Index], p.partition_number,
	p.[Rows], p.data_compression_desc, 
	fg.name as [Filegroup],
	sum(a.total_pages) as TotalPages, 
	sum(a.used_pages) as UsedPages, 
	sum(a.data_pages) as DataPages,
	sum(a.total_pages) * 8 as TotalSpaceKB, 
	sum(a.used_pages) * 8 as UsedSpaceKB, 
	sum(a.data_pages) * 8 as DataSpaceKB
from 
	sys.tables t with (nolock) join sys.indexes i with (nolock) on
		t.object_id = i.object_id
	join sys.partitions p with (nolock) on 
		i.object_id = p.object_id AND i.index_id = p.index_id
	join sys.allocation_units a with (nolock) on 
		p.partition_id = a.container_id
	join sys.filegroups fg with (nolock) on 
		a.data_space_id = fg.data_space_id
where
	t.object_id = object_id(N'Transactions2017')
group by
	t.object_id, i.index_id, i.name, p.partition_number,
	p.[Rows], p.data_compression_desc, fg.name
order by 
	i.index_id, p.partition_number;

select min(TranDate), max(TranDate)
from dbo.Transactions2017
where $Partition.pfTransactions2017(TranDate) = 1;
go

alter index IDX_Transactions2017_CustomerId
on dbo.Transactions2017 rebuild partition = 10
with (online = on);