/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                           Partitioned Tables                             */
/*                             Data Import                                  */
/****************************************************************************/

use master
go

alter database PartitionedDB set single_user with rollback immediate;

alter database PartitionedDB modify filegroup CCIOrders readwrite;
alter database PartitionedDB modify filegroup OldOrders readwrite;

alter database PartitionedDB set multi_user
go

use PartitionedDB
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='StagingOrders') drop table dbo.StagingOrders;
go

select max(OrderDate) from dbo.Orders2017_CCI;

create table dbo.StagingOrders
(
	OrderDate datetime2(0) not null,
	OrderId int not null,
	WarehouseId int not null,	
	CustomerId int not null, 
	ZipCode char(5) not null,
	Amount money not null,
	constraint CHK_StagingOrders
	check (OrderDate >= '2017-10-01' and OrderDate < '2017-11-01')
) on [CCIOrders];

insert into dbo.StagingOrders with (TABLOCKX)
		(OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount)
	select OrderDate, OrderId, WarehouseId, CustomerId, ZipCode, Amount
	from dbo.Orders2017_10
	where OrderId % 5 = 0;
go

create clustered columnstore index CCI_StagingOrders
on dbo.StagingOrders
with (data_compression = columnstore)
on [CCIOrders]
go

select count(*), min(OrderDate), max(OrderDate) from dbo.StagingOrders;
go

alter table dbo.Orders2017_CCI
drop constraint CHK_Orders_2017_CCI;
go

alter table dbo.StagingOrders
switch to dbo.Orders2017_CCI
partition 10;
go

alter table dbo.Orders2017_CCI
add constraint CHK_Orders_2017_CCI
check (OrderDate >= '2017-01-01' and OrderDate < '2017-11-01')
go

select count(*), min(OrderDate), max(OrderDate) from dbo.StagingOrders;

select max(OrderDate) from dbo.Orders2017_CCI;
select count(*) from dbo.Orders2017_CCI where $Partition.pfCCIOrders2017(OrderDate) = 10;