/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                           Partitioned Tables                             */
/*                      Sliding Window (Session 1)                          */
/****************************************************************************/

use PartitionedDB
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='SlidingWindowTmp') drop table dbo.SlidingWindowTmp;
if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='SlidingWindowDemo') drop table dbo.SlidingWindowDemo;
if exists(select * from sys.partition_schemes where name = 'psSlidingWindowDemo') drop partition scheme psSlidingWindowDemo;
if exists(select * from sys.partition_functions where name = 'pfSlidingWindowDemo') drop partition function pfSlidingWindowDemo;


create partition function pfSlidingWindowDemo(datetime2(0))
as range right for values
('2016-10-01','2016-11-01','2016-12-01','2017-01-01'
,'2017-02-01','2017-03-01','2017-04-01','2017-05-01'
,'2017-06-01','2017-07-01','2017-08-01','2017-09-01'
,'2017-10-01');

create partition scheme psSlidingWindowDemo 
as partition pfSlidingWindowDemo
all to ([RecentOrders]);
go

create table dbo.SlidingWindowDemo
(
	Id int not null,
	Val int not null,
	ADate datetime2(0) not null,
	Placeholder char(200),
) on psSlidingWindowDemo(ADate);

create clustered index IDX_SlidingWindowDemo_Id
on dbo.SlidingWindowDemo(Id,ADate)
on psSlidingWindowDemo(ADate);
go

create nonclustered index IDX_SlidingWindowDemo_Val
on dbo.SlidingWindowDemo(Val)
include(ADate)
on psSlidingWindowDemo(ADate);
go

insert into dbo.SlidingWindowDemo(ADate,Id, Val)
	select OrderDate, OrderId, OrderId
	from dbo.Orders2017_CCI
	where OrderId % 1000 = 0 or OrderDate >= '2017-10-01'
go

-- Locking / data movement
set statistics io, time on
begin tran
	alter partition function pfSlidingWindowDemo()
	split range ('2017-11-01');
	-- Run code from Session 2
commit
set statistics io, time off
go

-- Sliding window implementation 
create table dbo.SlidingWindowTmp
(
	Id int not null,
	Val int not null,
	ADate datetime2(0) not null,
	Placeholder char(200),
) on [RecentOrders];

create clustered index IDX_SlidingWindowTmp_Id
on dbo.SlidingWindowTmp(Id,ADate)
on [RecentOrders];
go

create nonclustered index IDX_SlidingWindowTmp_Val
on dbo.SlidingWindowTmp(Val)
include(ADate)
on [RecentOrders];
go

set xact_abort on
set statistics io, time on
begin tran
	alter table dbo.SlidingWindowDemo
	switch partition 1
	to dbo.SlidingWindowTmp;

	alter partition scheme psSlidingWindowDemo
	next used [RecentOrders];

	alter partition function pfSlidingWindowDemo()
	split range ('2017-12-01');
	-- Run code from Session 2
commit
set statistics io, time off

truncate table dbo.SlidingWindowTmp;
go



	