/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                           Partitioned Tables                             */
/*                           Hash Partitioning                              */
/****************************************************************************/

use PartitionedDB
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='IoTData_MoveMeToNoSQL') drop table dbo.IoTData_MoveMeToNoSQL;
if exists(select * from sys.partition_schemes where name = 'psIoT') drop partition scheme psIoT;
if exists(select * from sys.partition_functions where name = 'pfIoT') drop partition function pfIoT;

create partition function pfIoT(int)
as range right for values
(
	-1932735284,-1717986920,-1503238556,-1288490192
	,-1073741828,-858993464,-644245100,-429496736
	,-214748372,-8,214748356,429496720,644245084
	,858993448,1073741812,1288490176,1503238540
	,1717986904,1932735268,2147483632
);

create partition scheme psIoT
as partition pfIoT
all to ([RecentOrders]);

create table dbo.IoTData_MoveMeToNoSQL
(
	RecId bigint not null identity(1,1),
	HashKey int not null
		constraint DEF_IoTData_MoveMeToNoSQL
		default checksum(newid()),
	/* Other Columms */
);
go

create unique clustered index IDX_IoTData_MoveMeToNoSQL_RecId_HashKey
on dbo.IoTData_MoveMeToNoSQL(RecId,HashKey)
on psIoT(HashKey);
go

insert into dbo.IoTData_MoveMeToNoSQL 
	default values
go 1000

select 
	t.object_id, i.index_id, i.name as [Index], p.partition_number,
	p.[Rows], p.data_compression_desc, 
	fg.name as [Filegroup],
	sum(a.total_pages) as TotalPages, 
	sum(a.used_pages) as UsedPages, 
	sum(a.data_pages) as DataPages,
	sum(a.total_pages) * 8 as TotalSpaceKB, 
	sum(a.used_pages) * 8 as UsedSpaceKB, 
	sum(a.data_pages) * 8 as DataSpaceKB
from 
	sys.tables t with (nolock) join sys.indexes i with (nolock) on
		t.object_id = i.object_id
	join sys.partitions p with (nolock) on 
		i.object_id = p.object_id AND i.index_id = p.index_id
	join sys.allocation_units a with (nolock) on 
		p.partition_id = a.container_id
	join sys.filegroups fg with (nolock) on 
		a.data_space_id = fg.data_space_id
where
	t.object_id = object_id(N'IoTData_MoveMeToNoSQL')
group by
	t.object_id, i.index_id, i.name, p.partition_number,
	p.[Rows], p.data_compression_desc, fg.name
order by 
	i.index_id, p.partition_number;
go



/* Generating partition values 


declare
	@Min bigint = -2147483648
	,@Max bigint = 2147483647
	,@Values int = 20
	,@Step int

set @Step = power(2.0,32) / @Values


;with CTE(Value, Step)
as
(
	select @Min + @Step, 1
	union all 
	select Value + @Step, Step + 1
	from CTE
	where Step < @Values
)
select
	Value as [text()]
	,',' as [text()]
from CTE
order by Step
for xml path('') 

*/