/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                              Moving Data                                 */
/*                         Moving Regular Table                             */
/****************************************************************************/

use PartitionedDb
go

select 
	p.partition_number
	,object_name(object_id) as Name
	,filegroup_name(data_space_id) as FileGroup
	,type_desc
from 
	sys.partitions p join sys.allocation_units a on 
		p.partition_id = a.container_id
where 
	object_id = object_id('dbo.RegularTable')
order by 
	p.partition_number
go

create unique clustered index IDX_RegularTable_OrderDate_OrderId
on dbo.RegularTable(OrderDate, OrderId)
with (drop_existing = on, online = on)
on [RecentOrders]
go

select 
	p.partition_number
	,object_name(object_id) as Name
	,filegroup_name(data_space_id) as FileGroup
	,type_desc
from 
	sys.partitions p join sys.allocation_units a on 
		p.partition_id = a.container_id
where 
	object_id = object_id('dbo.RegularTable')
order by 
	p.partition_number
go

create partition function pfRegularTable(date)
as range right for values 
('2100-01-01')
go

alter partition function pfRegularTable()
merge range('2100-01-01')
go

create partition scheme psRegularTable
as partition pfRegularTable
all to ([RecentOrders])
go

create unique clustered index IDX_RegularTable_OrderDate_OrderId
on dbo.RegularTable(OrderDate, OrderId)
with (drop_existing = on, online = on)
on psRegularTable(OrderDate)
go

select 
	p.partition_number
	,object_name(object_id) as Name
	,filegroup_name(data_space_id) as FileGroup
	,type_desc
from 
	sys.partitions p join sys.allocation_units a on 
		p.partition_id = a.container_id
where 
	object_id = object_id('dbo.RegularTable')
order by 
	p.partition_number
go

