/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                        Going Beyond B-Tree                               */
/*                           Creating Tables                                */
/****************************************************************************/
set nocount on
set xact_abort on
go

use InMemoryOLTP2016
go

drop proc if exists dbo.InsertPayments2017_11;
drop proc if exists dbo.UpdatePayments2017_11;
drop proc if exists dbo.DeletePayments2017_11;
drop proc if exists dbo.UpdatePayments2017_10;
drop proc if exists dbo.InsertPayments2017_10;
drop proc if exists dbo.DeletePayments2017_10;
drop view if exists dbo.Payments;
drop table if exists dbo.Payments2017_11;
drop table if exists dbo.Payments2017_10;
drop table if exists dbo.Payments2017;
drop table if exists dbo.Payments2016;
drop table if exists dbo.Customers;
drop table if exists dbo.PaymentsUpdateQueue;
drop table if exists dbo.PaymentsDeleteQueue;
drop table if exists dbo.Payments2017_10_Tmp;
if exists(select * from sys.partition_schemes where name = 'ps2016') drop partition scheme ps2016;
if exists(select * from sys.partition_schemes where name = 'ps2017') drop partition scheme ps2017;
if exists(select * from sys.partition_functions where name = 'pf2016') drop partition function pf2016;
if exists(select * from sys.partition_functions where name = 'pf2017') drop partition function pf2017;
go

-- Catalog Entity
create table dbo.Customers
(
	CustomerId int not null
		constraint PK_Customers
		primary key nonclustered hash
		with (bucket_count=65536),
	Name nvarchar(256) not null, 

	index IDX_Customers_Name
	nonclustered(Name)
)
with (memory_optimized=on, durability=schema_and_data)
go

-- Storing data for 2017_11
create table dbo.Payments2017_11
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null,
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	/* Other columns */
	constraint PK_Payments2017_11
	primary key nonclustered hash(PaymentId)
	with (bucket_count=1048576),
	
	index IDX_Payments2017_11_CustomerId
	nonclustered hash(CustomerId)
	with (bucket_count=65536),

	constraint CHK_Payments2017_11
	check (PaymentDate >= '2017-11-01' and PaymentDate < '2017-12-01'),

	constraint FK_Payments2017_11_Customers
	foreign key(CustomerId)
	references dbo.Customers(CustomerId)
)
with (memory_optimized=on, durability=schema_and_data)
go

-- Storing data for 2017_10
create table dbo.Payments2017_10
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null,
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	/* Other columns */
	constraint PK_Payments2017_10
	primary key nonclustered hash(PaymentId)
	with (bucket_count=1048576),
	
	index IDX_Payments2017_10_CustomerId
	nonclustered hash(CustomerId)
	with (bucket_count=65536),

	constraint CHK_Payments2017_10
	check (PaymentDate >= '2017-10-01' and PaymentDate < '2017-11-01'),

	constraint FK_Payments2017_10_Customers
	foreign key(CustomerId)
	references dbo.Customers(CustomerId)
)
with (memory_optimized=on, durability=schema_and_data)
go



create partition function pf2017(datetime2(0))
as range right for values 
('2017-02-01','2017-03-01','2017-04-01','2017-05-01','2017-06-01','2017-07-01'
,'2017-08-01','2017-09-01','2017-10-01','2017-11-01','2017-12-01','2018-01-01');
go

create partition scheme ps2017
as partition pf2017
all to ([FG2017]);
go

-- Storing data for 2017
create table dbo.Payments2017
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null, 
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	constraint CHK_Payment2017_01_09 check (PaymentDate >= '2017-01-01' and PaymentDate < '2017-10-01'),
	constraint CHK_Payment2017_01_10 check (PaymentDate >= '2017-01-01' and PaymentDate < '2017-11-01'),
	constraint CHK_Payment2017_01_11 check (PaymentDate >= '2017-01-01' and PaymentDate < '2017-12-01'),
	constraint CHK_Payment2017 check (PaymentDate >= '2017-01-01' and PaymentDate < '2018-01-01'),
)
go

create unique clustered index IDX_Payments2017_PaymentDate_PaymentId
on dbo.Payments2017(PaymentDate, PaymentId)
with (data_compression=row)
on ps2017(PaymentDate)
go

create nonclustered index IDX_Payments2017_CustomerId
on  dbo.Payments2017(CustomerId)
with (data_compression=row)
on ps2017(PaymentDate)
go

create nonclustered index IDX_Payments2017_PaymentId
on  dbo.Payments2017(PaymentId)
with (data_compression=row)
on ps2017(PaymentDate)
go


create partition function pf2016(datetime2(0))
as range right for values 
('2016-02-01','2016-03-01','2016-04-01','2016-05-01','2016-06-01','2016-07-01'
,'2016-08-01','2016-09-01','2016-10-01','2016-11-01','2016-12-01','2017-01-01');
go

create partition scheme ps2016
as partition pf2016
all to ([FG2016]);
go

create table dbo.Payments2016
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null,
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	constraint CHK_Payment2016 check (PaymentDate >= '2016-01-01' and PaymentDate < '2017-01-01'),
)
on ps2016(PaymentDate)
go

create clustered columnstore index CCI_Payments2016
on dbo.Payments2016
on ps2016(PaymentDate)
go

create view dbo.Payments(PaymentDate, PaymentId, CustomerId, Amount, Status)
as
	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2017_11
	
	union all

	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2017_10

	union all

	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2017

	union all

	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2016
go


/* Populating data */
insert into dbo.Customers(CustomerId,Name)
	select Num, 'Customer ' + convert(varchar(6),Num)
	from dbo.Numbers
	where Num <= 16384;

insert into dbo.Payments2017_11(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(second,Num * 39,'2017-11-01') 
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 65536;

insert into dbo.Payments2017_10(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(second,Num * 39,'2017-10-01')
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 65536;

insert into dbo.Payments2017(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(minute,Num * 2,'2017-01-01')
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 196500;

insert into dbo.Payments2016(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(minute,Num * 7,'2016-01-01')
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 65536;
go

alter index CCI_Payments2016 on dbo.Payments2016 rebuild;
go

/* Execution Plans */
select count(*)
from dbo.Payments 
where PaymentDate between '2017-06-02' and '2017-06-03';

select count(*)
from dbo.Payments 
where PaymentDate >= '2017-01-01';

select count(*)
from dbo.Payments 
go

