/****************************************************************************/
/*      Everything You Always Wanted to Know about Data Partitioning        */
/*                         but Were AfraId to Ask                           */
/*                            PASS Summit 2017                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                               Challenges                                 */
/*                           Uniqueness Support                             */
/****************************************************************************/

use PartitionedDB
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_Id = s.schema_Id where s.name = 'dbo' and t.name='TmpTransactions') drop table dbo.TmpTransactions;
if object_id(N'dbo.GetTranCount','FN') is not null
begin
	alter table dbo.Transactions2017
	drop constraint CHK_TranNumberUniqueness;

	drop function dbo.GetTranCount;
end
go

-- This would fail:
create unique nonclustered index IDX_Transactions2017_TranNumber
on dbo.Transactions2017(TranNumber)
on psTransactions2017(TranDate);
go

-- This would succeed but index is not aligned:
create unique nonclustered  index IDX_Transactions2017_TranNumber
on dbo.Transactions2017(TranNumber)
on [RecentOrders]
--on psTransactions2017(TranDate);
go


create table dbo.TmpTransactions
(
	TranDate datetime2(0) not null,
	TranId int not null,
	TranNumber varchar(12) not null, 
	CustomerId int not null,
	Data varchar(1024) null,
) on [RecentOrders];

create unique clustered index IDX_TmpTransactions_TranId
on dbo.TmpTransactions(TranId)
on [RecentOrders];

create nonclustered index IDX_TmpTransactions_CustomerId
on dbo.TmpTransactions(CustomerId)
on [RecentOrders]
go

create unique nonclustered index IDX_TmpTransactions_TranNumber
on dbo.TmpTransactions(TranNumber)
on [RecentOrders];
go

-- It would fail
alter table dbo.Transactions2017
switch partition 1 
to dbo.TmpTransactions;
go

-- Workaround with performance implications
create nonclustered index IDX_Transactions2017_TranNumber
on dbo.Transactions2017(TranNumber)
with (drop_existing = on)
on psTransactions2017(TranDate);
go

create function dbo.GetTranCount(@TranNumber varchar(12))
returns int
with schemabinding
as
begin
	return
	(
		select count(*)
		from dbo.Transactions2017 with (readcommitted)
		where TranNumber = @TranNumber
	)
end
go

alter table dbo.Transactions2017
add constraint CHK_TranNumberUniqueness
check (dbo.GetTranCount(TranNumber) = 1);
go

select top 10 * from dbo.Transactions2017 order by TranNumber desc;

insert into dbo.Transactions2017(TranDate,TranId,TranNumber,CustomerId,Data)
values(getutcdate(),100000000,'9999900',1,'Data');
go

insert into dbo.Transactions2017(TranDate,TranId,TranNumber,CustomerId,Data)
values(getutcdate(),100000000,'9999999',1,'Data');
go

