/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                              Linked Servers                              */
/*                      Performance and Plan Cache Issues                   */
/****************************************************************************/

/****************************************************************************/
/* The script expects that PartitionedDB is created and populated with the  */
/* data and you have Linked Server setup with PartitioningExpress DB in place*/
/****************************************************************************/

set nocount on
go

use PartitionedDb
go

if exists(select * from sys.views v join sys.schemas s on v.schema_id = s.schema_id where s.name = 'dbo' and v.name='AllOrders') drop view dbo.AllOrders;
go

create view dbo.AllOrders(OrderDate,OrderId,StoreId,CustomerID,Region,Amount,Placeholder)
as
	select OrderDate,OrderId,StoreId,CustomerID,Region,Amount,Placeholder
	from dbo.Orders2016_10

	union all

	select OrderDate,OrderId,StoreId,CustomerID,Region,Amount,Placeholder
	from LINKEDSERVER.PartitioningExpress.dbo.OldOrders
	where OrderDate < '2016-10-01'
go


-- Look at execution plans
select count(*) 
from dbo.AllOrders
where StoreId = 10 and OrderDate < '2016-02-01'
go

select n.Number, count(*) 
from dbo.AllOrders o join dbo.Numbers n on
	n.Number = o.StoreId
where o.OrderDate < '2016-10-01' and n.number % 50 = 0
group by n.Number
go

-- Query would fail if plan is cached
declare
	@OrderDate datetime = '2016-10-29'

select count(*) 
from dbo.AllOrders 
where OrderDate = @OrderDate;



